@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,13 +20,10 @@
 That is import as expression, and star import.
 """
 
+from nuitka.PythonVersions import python_version
+
 from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
-from .ErrorCodes import (
-    getErrorExitBoolCode,
-    getErrorExitCode,
-    getReleaseCode,
-    getReleaseCodes
-)
+from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode
 from .LineNumberCodes import emitLineNumberUpdateCode
 from .ModuleCodes import getModuleAccessCode
 
@@ -66,7 +63,7 @@ def getCountedArgumentsHelperCallCode(helper_prefix, to_name, args, min_args,
                 to_name,
                 helper_prefix,
                 ", ".join(
-                    "NULL" if arg is None else arg
+                    "NULL" if arg is None else str(arg)
                     for arg in
                     orig_args
                 )
@@ -81,23 +78,22 @@ def getCountedArgumentsHelperCallCode(helper_prefix, to_name, args, min_args,
                 to_name,
                 helper_prefix,
                 len(args),
-                ", ".join(args)
+                ", ".join(
+                    str(arg)
+                    for arg in
+                    args
+                )
             )
         )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = args,
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -215,32 +211,27 @@ def generateImportStarCode(statement, emit, context):
             )
         )
     else:
-        locals_dict_name = statement.getLocalsDictScope().getCodeName()
+        locals_declaration = context.addLocalsDictName(
+            statement.getLocalsDictScope().getCodeName()
+        )
 
         emit(
             """
 %(res_name)s = IMPORT_MODULE_STAR( %(locals_dict)s, false, %(module_name)s );
 """ % {
                 "res_name"    : res_name,
-                "locals_dict" : locals_dict_name,
+                "locals_dict" : locals_declaration,
                 "module_name" : module_name
             }
         )
 
-        context.addLocalsDictName(locals_dict_name)
-
-    getReleaseCode(
+    getErrorExitBoolCode(
+        condition    = "%s == false" % res_name,
         release_name = module_name,
         emit         = emit,
         context      = context
     )
 
-    getErrorExitBoolCode(
-        condition = "%s == false" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
     context.setCurrentSourceCodeReference(old_source_ref)
 
 
@@ -254,27 +245,53 @@ def generateImportNameCode(to_name, expression, emit, context):
         context    = context
     )
 
-    emit(
-        "%s = IMPORT_NAME( %s, %s );" % (
-            to_name,
-            from_arg_name,
-            context.getConstantCode(
-                constant = expression.getImportName()
+    level = expression.getImportLevel()
+
+    if level and python_version >= 350:
+        emit(
+            """\
+if ( PyModule_Check( %(from_arg_name)s ) )
+{
+   %(to_name)s = IMPORT_NAME_OR_MODULE(
+        %(from_arg_name)s,
+        (PyObject *)MODULE_DICT(%(from_arg_name)s),
+        %(import_name)s,
+        %(import_level)s
+    );
+}
+else
+{
+   %(to_name)s = IMPORT_NAME( %(from_arg_name)s, %(import_name)s );
+}
+""" % {
+                "to_name"       : to_name,
+                "from_arg_name" : from_arg_name,
+                "import_name"   : context.getConstantCode(
+                    constant = expression.getImportName()
+                ),
+                "import_level"   : context.getConstantCode(
+                    constant = expression.getImportLevel()
+                )
+            }
+        )
+    else:
+        emit(
+            "%s = IMPORT_NAME( %s, %s );" % (
+                to_name,
+                from_arg_name,
+                context.getConstantCode(
+                    constant = expression.getImportName()
+                )
             )
         )
-    )
 
-    getReleaseCode(
+
+    getErrorExitCode(
+        check_name   = to_name,
         release_name = from_arg_name,
+        needs_check  = expression.mayRaiseException(BaseException),
         emit         = emit,
         context      = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = expression.mayRaiseException(BaseException),
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
