@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,24 +21,17 @@ Right now only the creation, and set add code is done here. But more should be
 added later on.
 """
 
-from .ErrorCodes import getErrorExitBoolCode, getReleaseCodes
-from .Helpers import generateChildExpressionsCode
+from nuitka.PythonVersions import needsSetLiteralReverseInsertion
+
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
+from .ErrorCodes import getErrorExitBoolCode
+from .PythonAPICodes import generateCAPIObjectCode
 
 
-def generateSetCreationCode(to_name, elements, emit, context):
-    emit(
-        "%s = PySet_New( NULL );" % (
-            to_name,
-        )
-    )
-
-    from .CodeGeneration import generateExpressionCode
-
-    context.addCleanupTempName(to_name)
-
+def generateSetCreationCode(to_name, expression, emit, context):
     element_name = context.allocateTempName("set_element")
 
-    for element in elements:
+    for count, element in enumerate(expression.getElements()):
         generateExpressionCode(
             to_name    = element_name,
             expression = element,
@@ -46,6 +39,69 @@ def generateSetCreationCode(to_name, elements, emit, context):
             context    = context
         )
 
+        if count == 0:
+            emit(
+                "%s = PySet_New( NULL );" % (
+                    to_name,
+                )
+            )
+
+            context.addCleanupTempName(to_name)
+
+        res_name = context.getIntResName()
+
+        emit(
+            "%s = PySet_Add( %s, %s );" % (
+                res_name,
+                to_name,
+                element_name
+            )
+        )
+
+        getErrorExitBoolCode(
+            condition   = "%s != 0" % res_name,
+            needs_check = not element.isKnownToBeHashable(),
+            emit        = emit,
+            context     = context
+        )
+
+        if context.needsCleanup(element_name):
+            emit("Py_DECREF( %s );" % element_name)
+            context.removeCleanupTempName(element_name)
+
+
+def generateSetLiteralCreationCode(to_name, expression, emit, context):
+    if not needsSetLiteralReverseInsertion():
+        return generateSetCreationCode(to_name, expression, emit, context)
+
+    emit(
+        "%s = PySet_New( NULL );" % (
+            to_name,
+        )
+    )
+
+    context.addCleanupTempName(to_name)
+
+    elements = expression.getElements()
+
+    element_names = []
+
+    for count, element in enumerate(elements):
+        element_name = context.allocateTempName(
+            "set_element_%d" % (count+1)
+        )
+        element_names.append(element_name)
+
+        generateExpressionCode(
+            to_name    = element_name,
+            expression = element,
+            emit       = emit,
+            context    = context
+        )
+
+    for count, element in enumerate(elements):
+        element_name = element_names[len(elements)-count-1]
+
         if element.isKnownToBeHashable():
             emit(
                 "PySet_Add( %s, %s );" % (
@@ -75,15 +131,27 @@ def generateSetCreationCode(to_name, elements, emit, context):
             context.removeCleanupTempName(element_name)
 
 
-def generateSetOperationAddCode(to_name, expression, emit, context):
-    res_name = context.getIntResName()
-
-    set_arg_name, value_arg_name = generateChildExpressionsCode(
-        expression = expression,
+def generateSetOperationAddCode(statement, emit, context):
+    set_arg_name = context.allocateTempName("append_list")
+    generateExpressionCode(
+        to_name    = set_arg_name,
+        expression = statement.getSet(),
         emit       = emit,
         context    = context
     )
 
+    value_arg_name = context.allocateTempName("append_value")
+    generateExpressionCode(
+        to_name    = value_arg_name,
+        expression = statement.getValue(),
+        emit       = emit,
+        context    = context
+    )
+
+    context.setCurrentSourceCodeReference(statement.getSourceReference())
+
+    res_name = context.getIntResName()
+
     emit("assert( PySet_Check( %s ) );" % set_arg_name)
     emit(
         "%s = PySet_Add( %s, %s );" % (
@@ -93,26 +161,13 @@ def generateSetOperationAddCode(to_name, expression, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == -1" % res_name,
         release_names = (set_arg_name, value_arg_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition = "%s == -1" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
-    # Only assign if necessary.
-    if context.isUsed(to_name):
-        emit(
-            "%s = Py_None;" % to_name
-        )
-    else:
-        context.forgetTempName(to_name)
-
 
 def generateSetOperationUpdateCode(to_name, expression, emit, context):
     res_name = context.getIntResName()
@@ -132,22 +187,41 @@ def generateSetOperationUpdateCode(to_name, expression, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == -1" % res_name,
         release_names = (set_arg_name, value_arg_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition = "%s == -1" % res_name,
-        emit      = emit,
-        context   = context
+    emit(
+        "%s = Py_None;" % to_name
     )
 
-    # Only assign if necessary.
-    if context.isUsed(to_name):
-        emit(
-            "%s = Py_None;" % to_name
-        )
-    else:
-        context.forgetTempName(to_name)
+
+def generateBuiltinSetCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "PySet_New",
+        arg_desc   = (
+            ("set_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
+
+
+def generateBuiltinFrozensetCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "PyFrozenSet_New",
+        arg_desc   = (
+            ("frozenset_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
